/*         
      L502 (   ).

       ,   
      .     
     (     ),   ' 
             .

       ,       Enter -  
       . 
   
         .
     ,          
   ,     L502_OutCycleSetup()  
            
        .


        Visual Studio 2008,     
   gcc  Linux  mingw  Windows  makefile    cmake (
       ).

         Visual Studio,     
   ( (Project) ->  (Properties) ->   (Configuration Properties)
   -> /++ ->  (General) ->    (Additional Include Directories))
    ,       l502api.h     
   ( (Project) ->  (Properties) ->   (Configuration Properties) ->
    (Linker) ->  (General) ->    (Additional Library Directories)).

   !!:      Visual Studio      (   SDK),
             
          UTF-8:
     (File) ->    (Advanced Save Options)...
       (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
       .
   */

#include "l502api.h"
#include "e502api.h"
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define TCP_CONNECTION_TOUT 5000
#define OUT_SIGNAL_SIZE           2000
#define OUT_BLOCK_SIZE            256
#define SEND_TOUT                 500

#ifndef M_PI
    #define M_PI 3.14159265358979323846
#endif

typedef double (*f_gen_sig_word)(uint32_t cntr, uint32_t total_size, double amp);
typedef uint32_t (*f_gen_dout_word)(uint32_t cntr, uint32_t total_size);
/* ,    2-   /1
1 DOUT */
typedef struct {
    uint32_t size; /*     */
    double amp_dac1; /*    1 */
    f_gen_sig_word gen_func_dac1; /*      1 */
    double amp_dac2; /*    2 */
    f_gen_sig_word gen_func_dac2; /*      2 */
    f_gen_dout_word gen_dout; /*        */
} t_sig_struct;


/*      */
static double f_gen_saw(uint32_t cntr, uint32_t total_size, double amp) {
    return amp*( (int32_t)(cntr%total_size)-(int32_t)total_size/2)/(total_size/2);
}

/*      */
static double f_gen_sin(uint32_t cntr, uint32_t total_size, double amp) {
    return amp*sin(2*M_PI*cntr/total_size);
}

/*             */
static double f_gen_sin2(uint32_t cntr, uint32_t total_size, double amp) {
    return amp*sin(2*2*M_PI*cntr/total_size);
}

/*             0xAA,    0x55 */
static uint32_t f_gen_dout_meander(uint32_t cntr, uint32_t total_size) {
    return cntr < total_size/2 ? 0xAA : 0x55;
}

/*     */
static uint32_t f_gen_dout_cntr(uint32_t cntr, uint32_t total_size) {
    return cntr;
}


/* ,       */
static t_sig_struct f_sig_tbl[] = {
    {100000, X502_DAC_RANGE, f_gen_saw, 0, NULL, NULL}, /*       10  */
    {2000, X502_DAC_RANGE, f_gen_sin, 0, NULL, NULL}, /*        500  */
    {100, X502_DAC_RANGE/2, f_gen_sin, X502_DAC_RANGE, f_gen_saw, NULL}, /*  . 2.5    10  */
    {50, 0, NULL, X502_DAC_RANGE, f_gen_sin, f_gen_dout_cntr}, /*        20  */
    {2550, 1.5, f_gen_sin, 2.5, f_gen_sin2, f_gen_dout_meander}, /*        2  .     */
};


/*         
   cntr -  ,    
   size -     (..  ch_cnt*size )
   sig  -  
   ch_cnt - -   ( -    f_sig_tbl[sig],
                  ,   ,     */
static int32_t f_load_block(t_x502_hnd hnd, uint32_t cntr, uint32_t size, uint32_t sig, uint32_t ch_cnt) {
    static double dac_data1[OUT_BLOCK_SIZE], dac_data2[OUT_BLOCK_SIZE];
    static uint32_t dout_data[OUT_BLOCK_SIZE];
    /*       -    
          (  3- - 2  + DOUT) */
    static uint32_t sbuf[3*OUT_BLOCK_SIZE];
    uint32_t i;
    int32_t err = 0;

    /*     */
    for (i=0; i < size; i++) {
        if (f_sig_tbl[sig].gen_func_dac1 != NULL) {
            dac_data1[i] = f_sig_tbl[sig].gen_func_dac1(cntr+i, f_sig_tbl[sig].size, f_sig_tbl[sig].amp_dac1);
        }
        if (f_sig_tbl[sig].gen_func_dac2 != NULL) {
            dac_data2[i] = f_sig_tbl[sig].gen_func_dac2(cntr+i, f_sig_tbl[sig].size, f_sig_tbl[sig].amp_dac2);
        }
        if (f_sig_tbl[sig].gen_dout != NULL) {
            dout_data[i] = f_sig_tbl[sig].gen_dout(cntr+i, f_sig_tbl[sig].size);
        }
    }

    /*    ,     , 
     *     .  -   
     *     NULL */
    err = X502_PrepareData(hnd,
                           f_sig_tbl[sig].gen_func_dac1 ? dac_data1 : NULL,
                           f_sig_tbl[sig].gen_func_dac2 ? dac_data2 : NULL,
                           f_sig_tbl[sig].gen_dout ? dout_data : NULL,
                           size, X502_DAC_FLAGS_VOLT | X502_DAC_FLAGS_CALIBR,
                           sbuf);
    if (err != X502_ERR_OK) {
        fprintf(stderr, "    : %s\n",
                X502_GetErrorString(err));
    } else {
        /*   */
        int32_t snd_cnt = X502_Send(hnd, sbuf, size*ch_cnt, SEND_TOUT);
        if (snd_cnt < 0) {
            err = snd_cnt;
            fprintf(stderr, "  : %s\n", X502_GetErrorString(err));
        } else if ((uint32_t)snd_cnt != size*ch_cnt) {
            /*         ,  
                  */
            fprintf(stderr, "  :  = %d,  = %d\n",
                    size*ch_cnt, snd_cnt);
            err = X502_ERR_SEND_INSUFFICIENT_WORDS;
        }
    }
    return err;
}

static int32_t f_load_cycle_signal(t_x502_hnd hnd, int sig) {
    int32_t err = 0;
    uint32_t cntr = 0;
    uint32_t ch_cnt=0;

    /* ,    */
    if (f_sig_tbl[sig].gen_func_dac1)
        ch_cnt++;
    if (f_sig_tbl[sig].gen_func_dac2)
        ch_cnt++;
    if (f_sig_tbl[sig].gen_dout)
        ch_cnt++;


    /*      ! */
    err = X502_OutCycleLoadStart(hnd, f_sig_tbl[sig].size*ch_cnt);
    if (err != X502_ERR_OK)
        fprintf(stderr, "   : %s!", X502_GetErrorString(err));

    /*   ,    ,   
     *        .   
     *      L502_Send() */
    while ((cntr != f_sig_tbl[sig].size) && (err == X502_ERR_OK)) {
        uint32_t block_size = OUT_BLOCK_SIZE;
        /*      ,     
         *  */
        if (block_size >(f_sig_tbl[sig].size-cntr))
            block_size=f_sig_tbl[sig].size-cntr;

        err = f_load_block(hnd, cntr, block_size, sig, ch_cnt);
        if (!err)
            cntr+=block_size;
    }

    /*     */
    if (err == X502_ERR_OK) {
       err = X502_OutCycleSetup(hnd, X502_OUT_CYCLE_FLAGS_WAIT_DONE);
       if (err != X502_ERR_OK)
            fprintf(stderr, "   : %s!", X502_GetErrorString(err));
    }
    return err;
}


/*        PCI-Express  USB 
 *        .
 *      IP-     
 * .
 *    ,     , 
 *  pdevrec_list,     (  
 *       X502_FreeDevRecordList()) 
 *    */
static uint32_t f_get_all_devrec(t_x502_devrec **pdevrec_list, uint32_t *ip_addr_list, unsigned ip_cnt) {
    int32_t fnd_devcnt = 0;
    uint32_t pci_devcnt = 0;
    uint32_t usb_devcnt = 0;

    t_x502_devrec *rec_list = NULL;

    /*       PCI  USB */
    L502_GetDevRecordsList(NULL, 0, 0, &pci_devcnt);
    E502_UsbGetDevRecordsList(NULL, 0, 0, &usb_devcnt);

    if ((pci_devcnt+usb_devcnt + ip_cnt) != 0) {
        /*          */
        rec_list = malloc((pci_devcnt + usb_devcnt + ip_cnt) * sizeof(t_x502_devrec));

        if (rec_list != NULL) {
            unsigned i;
            /*     L502,    pci_devcnt */
            if (pci_devcnt!=0) {
                int32_t res = L502_GetDevRecordsList(&rec_list[fnd_devcnt], pci_devcnt, 0, NULL);
                if (res >= 0) {
                    fnd_devcnt += res;
                }
            }
            /*     E502,   USB,    */
            if (usb_devcnt!=0) {
                int32_t res = E502_UsbGetDevRecordsList(&rec_list[fnd_devcnt], usb_devcnt, 0, NULL);
                if (res >= 0) {
                    fnd_devcnt += res;
                }
            }

            /*      ip- */
            for (i=0; i < ip_cnt; i++) {
                if (E502_MakeDevRecordByIpAddr(&rec_list[fnd_devcnt], ip_addr_list[i],0, TCP_CONNECTION_TOUT) == X502_ERR_OK) {
                    fnd_devcnt++;
                }
            }
        }
    }

    if (fnd_devcnt != 0) {
        /*      ,       */
        *pdevrec_list = rec_list;
    } else {
        *pdevrec_list = NULL;
        free(rec_list);
    }

    return fnd_devcnt;
}


static t_x502_hnd f_dev_select_open(int argc, char** argv) {
    t_x502_hnd hnd = NULL;
    uint32_t fnd_devcnt,i, dev_ind;
    t_x502_devrec *devrec_list = NULL;
    uint32_t *ip_addr_list = NULL;
    uint32_t ip_cnt = 0;

    /*     ,  ,    
       ip-  . */
    if (argc > 1) {
        ip_addr_list = malloc((argc-1) * sizeof(ip_addr_list[0]));
        if (ip_addr_list == NULL) {
            fprintf(stderr, "  !\n");
        } else {
            for (i=1; (int)i < argc; i++) {
                int a[4];
                if (sscanf(argv[i], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])==4) {
                    ip_addr_list[ip_cnt++] = ((a[0] & 0xFF) << 24) |
                                             ((a[1] & 0xFF) << 16) |
                                             ((a[2] & 0xFF) <<  8) |
                                             (a[3] & 0xFF);
                }
            }
        }
    }

    /*      */
    fnd_devcnt = f_get_all_devrec(&devrec_list, ip_addr_list, ip_cnt);

    if (fnd_devcnt == 0) {
        printf("    \n");
    } else {
        /*      */
        printf("  :\n");
        for (i=0; i < fnd_devcnt; i++) {
            printf("  %d: %s, %-9s", i, devrec_list[i].devname,
                   devrec_list[i].iface == X502_IFACE_PCI ? "PCI/PCIe" :
                   devrec_list[i].iface == X502_IFACE_USB ? "USB" :
                   devrec_list[i].iface == X502_IFACE_ETH ? "Ethernet" : "?");

            /*      IP-    
                  .    location
                     */
            if (devrec_list[i].iface != X502_IFACE_ETH) {
                printf(". : %s\n", devrec_list[i].serial);
            } else {
                printf(": %s\n", devrec_list[i].location);
            }
        }

        /*           */
        printf("  ,     ( 0  %d)\n", fnd_devcnt-1);
        fflush(stdout);
        scanf("%d", &dev_ind);

        if (dev_ind >= fnd_devcnt) {
            printf("   ...\n");
        } else {
            /*     -   */
            hnd = X502_Create();
            if (hnd==NULL) {
                fprintf(stderr, "   !");
            } else {
                /*       */
                int32_t err = X502_OpenByDevRecord(hnd, &devrec_list[dev_ind]);
                if (err != X502_ERR_OK) {
                    fprintf(stderr, "    : %s!", X502_GetErrorString(err));
                    X502_Free(hnd);
                    hnd = NULL;
                }
            }
        }

        /*       */
        X502_FreeDevRecordList(devrec_list, fnd_devcnt);
        /*     */
        free(devrec_list);
    }

    /*     IP- (  ) */
    free(ip_addr_list);

    return hnd;
}

int main(int argc, char **argv) {
    int32_t err = 0;
    uint32_t ver;
    t_x502_hnd hnd = NULL;

#ifdef _WIN32
    /*  ,     -  CP1251    OEM */
    setlocale(LC_CTYPE, "");
#endif
    /*    */
    ver = X502_GetLibraryVersion();
    printf(" : %d.%d.%d\n", (ver >> 24)&0xFF, (ver>>16)&0xFF, (ver>>8)&0xFF);

    /**********     ,     ******************/
    hnd = f_dev_select_open(argc, argv);

    /**********************************    **************************/
    /*          -   */
    if (hnd != NULL) {
        /*   */
        t_x502_info info;
        err = X502_GetDevInfo(hnd, &info);
        if (err != X502_ERR_OK) {
            fprintf(stderr, "     : %s!", X502_GetErrorString(err));
        } else {
            /*    */
            printf("    :\n");
            printf("            : %s\n", info.serial);
            printf("               : %s\n", info.devflags & X502_DEVFLAGS_DAC_PRESENT ? "" : "");
            printf("  BlackFin        : %s\n", info.devflags & X502_DEVFLAGS_BF_PRESENT ? "" : "");
            printf("  : %s\n", info.devflags & X502_DEVFLAGS_GAL_PRESENT ? "" : "");
            printf("  .     : %s\n", info.devflags & X502_DEVFLAGS_INDUSTRIAL ? "" : "");
            printf("  . PCI/PCIe: %s\n", info.devflags & X502_DEVFLAGS_IFACE_SUPPORT_PCI ? "" : "");
            printf("  . USB     : %s\n", info.devflags & X502_DEVFLAGS_IFACE_SUPPORT_USB ? "" : "");
            printf("  . Ethernet: %s\n", info.devflags & X502_DEVFLAGS_IFACE_SUPPORT_ETH ? "" : "");
            printf("               : %d.%d\n", (info.fpga_ver >> 8) & 0xFF, info.fpga_ver & 0xFF);
            printf("  PLDA             : %d\n", info.plda_ver);
            if (info.mcu_firmware_ver != 0) {
                printf("   ARM     : %d.%d.%d.%d\n",
                       (info.mcu_firmware_ver >> 24) & 0xFF,
                       (info.mcu_firmware_ver >> 16) & 0xFF,
                       (info.mcu_firmware_ver >>  8) & 0xFF,
                       info.mcu_firmware_ver & 0xFF);
            }
        }

        if (err == X502_ERR_OK) {
            /*      (     
                 ),     
                     L502_PrepareData */
            err = X502_StreamsEnable(hnd, X502_STREAM_ALL_OUT);
            if (err != X502_ERR_OK) {
                fprintf(stderr, "   (%d): %s!", err,
                        X502_GetErrorString(err));
            }
        }

        if (err == X502_ERR_OK) {
            /*  ,      
             *  .      , ,
                   L502_StreamsStart() */
            err = f_load_cycle_signal(hnd, 0);

            /*   .      
             *       .   
             *          ,
             *           
             *      */
            if (err == X502_ERR_OK)
                err = X502_StreamsStart(hnd);

            if (err == X502_ERR_OK) {
                int exit = 0;

                printf("   :\n");
                printf("    1  %d -     \n",
                       (int)(sizeof(f_sig_tbl)/sizeof(f_sig_tbl[0])));
                printf("  s  stop       -   \n");
                printf("  e  exit       -   \n");

                /*    */
                while (!err && !exit) {
                    char cmd[512];
                    printf(">");
                    fflush(stdout);
                    scanf("%s", cmd);
                    if (!strcmp(cmd, "exit") || !strcmp(cmd,"e")) {
                        exit=1;
                    } else if (!strcmp(cmd, "stop")||!strcmp(cmd,"s")) {
                        err = X502_OutCycleStop(hnd, X502_OUT_CYCLE_FLAGS_WAIT_DONE);
                        if (err != X502_ERR_OK) {
                            fprintf(stderr, "    (%d): %s\n",
                                    err, X502_GetErrorString(err));
                        }
                    } else {
                        int sig = atoi(cmd);
                        if ((sig <= 0) || (sig > (int)(sizeof(f_sig_tbl)/sizeof(f_sig_tbl[0])))) {
                            fprintf(stderr, "     \n");
                        } else {
                            err = f_load_cycle_signal(hnd, sig-1);
                        }
                    }
                }

                X502_StreamsStop(hnd);
            }            
        }

        /*     */
        X502_Close(hnd);
        /*   */
        X502_Free(hnd);
    }
    return err;
}
  

